"use strict";
/* eslint-disable security/detect-object-injection */
/* eslint-disable prettier/prettier */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqliteColumnCompiler = exports.SqliteInsertQueryCompiler = exports.SqliteTableQueryCompiler = exports.SqliteTableExistsCompiler = exports.SqliteOnDuplicateQueryCompiler = exports.SqliteOrderByCompiler = exports.SqliteTruncateTableQueryCompiler = void 0;
const orm_sql_1 = require("@spinajs/orm-sql");
const orm_1 = require("@spinajs/orm");
const di_1 = require("@spinajs/di");
const lodash_1 = __importDefault(require("lodash"));
let SqliteTruncateTableQueryCompiler = class SqliteTruncateTableQueryCompiler extends orm_1.TableQueryCompiler {
    constructor(container, builder) {
        super();
        this.container = container;
        this.builder = builder;
    }
    compile() {
        return {
            bindings: [],
            expression: `DELETE FROM ${this.container.resolve(orm_1.TableAliasCompiler).compile(this.builder)}`,
        };
    }
};
exports.SqliteTruncateTableQueryCompiler = SqliteTruncateTableQueryCompiler;
exports.SqliteTruncateTableQueryCompiler = SqliteTruncateTableQueryCompiler = __decorate([
    (0, di_1.NewInstance)(),
    __metadata("design:paramtypes", [di_1.Container, orm_1.TableQueryBuilder])
], SqliteTruncateTableQueryCompiler);
let SqliteOrderByCompiler = class SqliteOrderByCompiler extends orm_1.OrderByQueryCompiler {
    constructor(builder) {
        super();
        if (!builder) {
            throw new Error('builder cannot be null or undefined');
        }
        this._builder = builder;
    }
    compile() {
        const sort = this._builder.getSort();
        let stmt = '';
        const bindings = [];
        if (sort) {
            stmt = ` ORDER BY \`${sort.column}\` ${sort.order.toLowerCase() === 'asc' ? 'ASC' : 'DESC'}`;
        }
        return {
            bindings,
            expression: stmt,
        };
    }
};
exports.SqliteOrderByCompiler = SqliteOrderByCompiler;
exports.SqliteOrderByCompiler = SqliteOrderByCompiler = __decorate([
    (0, di_1.NewInstance)(),
    __metadata("design:paramtypes", [orm_1.OrderByBuilder])
], SqliteOrderByCompiler);
let SqliteOnDuplicateQueryCompiler = class SqliteOnDuplicateQueryCompiler extends orm_sql_1.SqlOnDuplicateQueryCompiler {
    constructor(builder) {
        super(builder);
    }
    compile() {
        if (this._builder.getColumn().length === 0) {
            throw new orm_1.OrmException(`no unique or primary key columns defined in table ${this._builder.getParent().Table}`);
        }
        const columns = this._builder.getColumnsToUpdate().map((c) => {
            if (lodash_1.default.isString(c)) {
                return `${c} = ?`;
            }
            else {
                return c.Query;
            }
        });
        const parent = this._builder.getParent();
        const bindings = lodash_1.default.flatMap(this._builder.getColumnsToUpdate(), (c) => {
            if (lodash_1.default.isString(c)) {
                const cIndex = parent.getColumns().findIndex((col) => (lodash_1.default.isString(col.Column) ? col.Column === c : null));
                return parent.Values[0][cIndex];
            }
            else {
                return c.Bindings;
            }
        });
        const returning = this._builder.getReturning()[0] === '*' ? ['*'] : this._builder.getReturning().map((c) => `\`${c}\``);
        return {
            bindings,
            expression: `ON CONFLICT(${this._builder.getColumn().join(',')}) DO UPDATE SET ${columns.join(',')} RETURNING ${returning.join(',')};`,
        };
    }
};
exports.SqliteOnDuplicateQueryCompiler = SqliteOnDuplicateQueryCompiler;
exports.SqliteOnDuplicateQueryCompiler = SqliteOnDuplicateQueryCompiler = __decorate([
    (0, di_1.NewInstance)(),
    __metadata("design:paramtypes", [orm_1.OnDuplicateQueryBuilder])
], SqliteOnDuplicateQueryCompiler);
let SqliteTableExistsCompiler = class SqliteTableExistsCompiler {
    constructor(builder) {
        this.builder = builder;
        if (builder === null) {
            throw new Error('table exists query builder cannot be null');
        }
    }
    compile() {
        return {
            bindings: [this.builder.Table],
            expression: `SELECT name FROM sqlite_master WHERE type='table' AND name=? LIMIT 1;`,
        };
    }
};
exports.SqliteTableExistsCompiler = SqliteTableExistsCompiler;
exports.SqliteTableExistsCompiler = SqliteTableExistsCompiler = __decorate([
    (0, di_1.NewInstance)(),
    __metadata("design:paramtypes", [orm_1.TableExistsQueryBuilder])
], SqliteTableExistsCompiler);
let SqliteTableQueryCompiler = class SqliteTableQueryCompiler extends orm_sql_1.SqlTableQueryCompiler {
    compile() {
        const _table = this._table();
        const _columns = this._columns();
        const _foreignKeys = this._foreignKeys();
        return [
            {
                bindings: [],
                expression: `${_table} (${_columns} ${_foreignKeys ? ',' + _foreignKeys : ''})`,
            },
        ];
    }
};
exports.SqliteTableQueryCompiler = SqliteTableQueryCompiler;
exports.SqliteTableQueryCompiler = SqliteTableQueryCompiler = __decorate([
    (0, di_1.NewInstance)(),
    (0, di_1.Inject)(di_1.Container)
], SqliteTableQueryCompiler);
let SqliteInsertQueryCompiler = class SqliteInsertQueryCompiler extends orm_sql_1.SqlInsertQueryCompiler {
    constructor(container, builder) {
        super(container, builder);
    }
    compile() {
        const into = this.into();
        const columns = this.columns();
        const values = this.values();
        const upsort = this.upsort();
        return {
            bindings: values.bindings.concat(upsort.bindings),
            expression: `${into} ${columns} ${values.data} ${upsort.expression}`.trim(),
        };
    }
    into() {
        return `INSERT${this._builder.Ignore ? ' OR IGNORE' : ''}${this._builder.Replace ? ' OR REPLACE' : ''} INTO \`${this._builder.Table}\``;
    }
};
exports.SqliteInsertQueryCompiler = SqliteInsertQueryCompiler;
exports.SqliteInsertQueryCompiler = SqliteInsertQueryCompiler = __decorate([
    (0, di_1.NewInstance)(),
    (0, di_1.Inject)(di_1.Container),
    __metadata("design:paramtypes", [Object, orm_1.InsertQueryBuilder])
], SqliteInsertQueryCompiler);
let SqliteColumnCompiler = class SqliteColumnCompiler extends orm_sql_1.SqlColumnQueryCompiler {
    compile() {
        const _stmt = [];
        _stmt.push(`\`${this.builder.Name}\``);
        switch (this.builder.Type) {
            case 'binary':
            case 'tinyblob':
            case 'mediumblob':
            case 'longblob':
                _stmt.push('BLOB');
                break;
            case 'string':
            case 'text':
            case 'mediumtext':
            case 'tinytext':
            case 'longtext':
            case 'date':
            case 'dateTime':
            case 'time':
            case 'set':
            case 'timestamp':
            case 'enum':
                _stmt.push(`TEXT`);
                break;
            case 'float':
            case 'double':
                _stmt.push(`REAL`);
                break;
            case 'decimal':
                _stmt.push(`DECIMAL`);
                break;
            case 'int':
            case 'tinyint':
            case 'smallint':
            case 'mediumint':
            case 'bigint':
                _stmt.push('INTEGER');
                break;
            case 'boolean':
                _stmt.push(`BOOLEAN NOT NULL CHECK ( \`${this.builder.Name}\` IN (0, 1))`);
                break;
        }
        if (this.builder.Unsigned) {
            _stmt.push('UNSIGNED');
        }
        if (this.builder.Charset) {
            _stmt.push(`CHARACTER SET '${this.builder.Charset}'`);
        }
        if (this.builder.Collation) {
            _stmt.push(`COLLATE '${this.builder.Collation}'`);
        }
        if (this.builder.NotNull) {
            _stmt.push('NOT NULL');
        }
        if (this.builder.Default) {
            _stmt.push(this._defaultCompiler());
        }
        if (this.builder.Comment) {
            _stmt.push(`COMMENT '${this.builder.Comment}'`);
        }
        if (this.builder.PrimaryKey) {
            _stmt.push(`PRIMARY KEY`);
        }
        if (this.builder.AutoIncrement) {
            _stmt.push(`AUTOINCREMENT`);
        }
        if (this.builder.Unique) {
            _stmt.push('UNIQUE');
        }
        return {
            bindings: [],
            expression: _stmt.filter((x) => !lodash_1.default.isEmpty(x)).join(' '),
        };
    }
    _defaultCompiler() {
        let _stmt = '';
        if (lodash_1.default.isNil(this.builder.Default) || (lodash_1.default.isString(this.builder.Default) && lodash_1.default.isEmpty(this.builder.Default.trim()))) {
            return _stmt;
        }
        if (lodash_1.default.isString(this.builder.Default.Value)) {
            _stmt = `DEFAULT '${this.builder.Default.Value.trim()}'`;
        }
        else if (lodash_1.default.isNumber(this.builder.Default.Value)) {
            _stmt = `DEFAULT ${this.builder.Default.Value}`;
        }
        else if (this.builder.Default.Query instanceof orm_1.RawQuery) {
            _stmt = `DEFAULT (${this.builder.Default.Query.Query})`;
        }
        return _stmt;
    }
};
exports.SqliteColumnCompiler = SqliteColumnCompiler;
exports.SqliteColumnCompiler = SqliteColumnCompiler = __decorate([
    (0, di_1.NewInstance)()
], SqliteColumnCompiler);
//# sourceMappingURL=compilers.js.map